@@ -34,7 +34,7 @@ from Crypto.Util._raw_api import (load_pycryptodome_raw_lib,
                                   VoidPointer, SmartPointer,
                                   create_string_buffer,
                                   get_raw_buffer, c_size_t,
-                                  expect_byte_string)
+                                  c_uint8_ptr)
 
 from Crypto.Hash.keccak import _raw_keccak_lib
 
@@ -68,15 +68,14 @@ class SHAKE128_XOF(object):
         """Continue hashing of a message by consuming the next chunk of data.
 
         Args:
-            data (byte string): The next chunk of the message being hashed.
+            data (byte string/byte array/memoryview): The next chunk of the message being hashed.
         """
 
         if self._is_squeezing:
             raise TypeError("You cannot call 'update' after the first 'read'")
 
-        expect_byte_string(data)
         result = _raw_keccak_lib.keccak_absorb(self._state.get(),
-                                               data,
+                                               c_uint8_ptr(data),
                                                c_size_t(len(data)))
         if result:
             raise ValueError("Error %d while updating SHAKE128 state"
@@ -117,7 +116,7 @@ def new(data=None):
     """Return a fresh instance of a SHAKE128 object.
 
     Args:
-       data (byte string):
+       data (byte string/byte array/memoryview):
         The very first chunk of the message to hash.
         It is equivalent to an early call to :meth:`update`.
         Optional.
